﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ServiceModel;
using System.Activities.Tracking;
using System.ServiceModel.Web;
using System.ServiceModel.Syndication;
using System.ServiceModel.Activation;

namespace AdvDataServices
{
    [ServiceContract()]
    [AspNetCompatibilityRequirements(RequirementsMode=AspNetCompatibilityRequirementsMode.Allowed)]
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class ActivatedUsersFeed
    {
        const int queueSize = 10;
        List<TrackingRecord> records = new List<TrackingRecord>(queueSize);

        [OperationContract]
        [WebInvoke(Method="POST", UriTemplate="/records",RequestFormat=WebMessageFormat.Xml, BodyStyle=WebMessageBodyStyle.Bare)]
        [ServiceKnownType(typeof(ActivityStateRecord))]
        public void SubmitTracking(TrackingRecord record)
        {
            lock (records)
            {
                if (records.Count == queueSize)
                    records.Remove(
                        records.Where(r => r.EventTime == records.Min(rd => rd.EventTime)).First());

                records.Add(record);
            }
        }

        [OperationContract]
        [WebGet(UriTemplate = "/records")]
        public Atom10FeedFormatter GetFeed()
        {

            SyndicationFeed feed =
                new SyndicationFeed
                {
                    Title = new TextSyndicationContent("Recent user feed"),
                    Description = new TextSyndicationContent("Provides a list of the most recent users to be activated in the system."),
                    Items =
                        from i in records
                        select new SyndicationItem
                        {
                            PublishDate = i.EventTime,
                            Title = new TextSyndicationContent(((ActivityStateRecord)i).Arguments["UserName"].ToString())
                        }
                };

            return new Atom10FeedFormatter(feed);
                     
        }
    }
}