﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using FinancialAppWeb.Services;
using System.Threading;
using FinancialAppWeb.Filters;

namespace FinancialAppWeb.Controllers
{
    public class ApplicationController : Controller
    {
        //
        // GET: /Application/

        [HttpGet]
        [Authorize]
        [WorkflowBookmarkFilter]
        public ActionResult Index()
        {
            return View();
        }

        // submitting initial data
        [HttpPost]
        [Authorize]
        public ActionResult Index(Models.PersonalInfoModel model)
        {
            if (ModelState.IsValid)
            {
                WorkflowService.StartWorkflow<CustomActivities.ApplicationProcessing>(
                   new Dictionary<string, object>{
                    {"userName", this.HttpContext.User.Identity.Name}});
                Thread.Sleep(1000);
                WorkflowService.SendDataToWorkflow<CustomActivities.ApplicationProcessing, Contracts.PersonalInformation>(
                    new Contracts.PersonalInformation {FirstName  = model.FirstName, LastName=model.LastName }, "Index");

                return RedirectToAction("Financials");
            }
            return View(model);
        }


        [HttpGet]
        [Authorize]
        [WorkflowBookmarkFilter]
        public ActionResult Financials()
        {
            return View(new Models.FinancialsModel());
        }

        [HttpPost]
        [Authorize]
        public ActionResult Financials(Models.FinancialsModel model)
        {
            if (ModelState.IsValid)
            {
                WorkflowService.SendDataToWorkflow<CustomActivities.ApplicationProcessing, Contracts.FinancialData>(
                    new Contracts.FinancialData { Money = model.HowMuchMoney }, "Financials");

               return RedirectToAction("EmploymentHistory");
            }
            return View(model);
            
        }

        [HttpGet]
        [Authorize]
        [WorkflowBookmarkFilter]
        public ActionResult EmploymentHistory()
        {
            return View(new Models.EmploymentHistoryModel());
        }

        [HttpPost]
        [Authorize]
        public ActionResult EmploymentHistory(Models.EmploymentHistoryModel model)
        {
            if (ModelState.IsValid)
            {
                WorkflowService.SendDataToWorkflow<CustomActivities.ApplicationProcessing, Contracts.EmploymentHistory>(
                    new Contracts.EmploymentHistory {HasJob = model.HasJob }, "EmploymentHistory");

                return RedirectToAction("Index", "Home");
            }
            return View(model);
        }

    }
}
