﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Activities;
using System.Activities.DurableInstancing;
using System.Data.SqlClient;
using System.Configuration;
using System.Diagnostics;
using System.Runtime.DurableInstancing;
using System.Threading;

namespace FinancialAppWeb.Services
{
    public class WorkflowService
    {

        internal static void StartWorkflow<TWorkflow >(Dictionary<string, object> inputs)  where TWorkflow :  System.Activities.Activity,new()
        {
            ManualResetEventSlim waitHandle =
               new ManualResetEventSlim(false);

            WorkflowApplication application =
                new WorkflowApplication(new TWorkflow(),
                    inputs);
            application.Extensions.Add(
                new CustomActivities.UserWorkflowPersistenceParticipant());
            application.Extensions.Add(
                new CustomTrackingService());
            application.Extensions.Add(
                new System.Activities.Tracking.EtwTrackingParticipant());


            application.InstanceStore  = WorkflowOwnerShipService.GetStore(Guid.Empty);

            
            application.PersistableIdle = (wiea) =>
            {
                waitHandle.Set();
                return PersistableIdleAction.Unload;
            };
            application.OnUnhandledException += (ea) =>
            {
                waitHandle.Set();
                Debug.WriteLine(ea.UnhandledException.Message);
                return UnhandledExceptionAction.Terminate;
            };

            application.Run();
            waitHandle.Wait(2000);

        }

        //private static void unloadComplete(IAsyncResult result)
        //{
        //    Action<WorkflowApplication> unloadAction =
        //        result.AsyncState as Action<WorkflowApplication>;
        //    unloadAction.EndInvoke(result);

        //}

        internal static void SendDataToWorkflow<TWorkflow, TData>(TData data, string dataKey, string userEmail) where TWorkflow : Activity, new()
        {
            ManualResetEventSlim waitHandle =
                new ManualResetEventSlim(false);

            WorkflowApplication application =
                new WorkflowApplication(new TWorkflow());
            application.Extensions.Add(
                new CustomTrackingService());
            application.Extensions.Add(
                new System.Activities.Tracking.EtwTrackingParticipant());

            application.PersistableIdle = (wiea) =>
            {
                waitHandle.Set();
                return PersistableIdleAction.Unload;
            };
            
            application.OnUnhandledException += (ea) =>
                {
                    waitHandle.Set();
                    Debug.WriteLine(ea.UnhandledException.Message);
                    return UnhandledExceptionAction.Terminate;
                };
            Guid wfID = GetWorkflowForUser(typeof(TWorkflow).Name, 
                userEmail ?? HttpContext.Current.User.Identity.Name);
            application.Extensions.Add(
                new CustomActivities.UserWorkflowPersistenceParticipant());

            application.InstanceStore = WorkflowOwnerShipService.GetStore(wfID);
            application.Load(wfID);
            application.ResumeBookmark(dataKey, data);
            waitHandle.Wait(2000);
        }

        internal static void SendDataToWorkflow<TWorkflow, TData>(TData data, string dataKey) where TWorkflow : Activity, new()
        {
            SendDataToWorkflow<TWorkflow, TData>(data, dataKey, null); 
        }

        private static Guid GetWorkflowForUser(string workflowTypeName,string userName)
        {
         	// load from the database based on keys
            using (SqlConnection cnn = new SqlConnection(
                ConfigurationManager.ConnectionStrings[Constants.CONN_PERSISTENCE].ConnectionString))
            {
                SqlCommand cmd = new SqlCommand(
                   String.Format( @"SELECT instanceid FROM [System.Activities.DurableInstancing].[InstancePromotedProperties] p
                where p.PromotionName='{0}' and p.Value1 = '{1}' and p.Value2 = '{2}'", 
                Constants.PROMOTION_NAME,workflowTypeName, userName),
                    cnn);
                cnn.Open();
                object wfid = cmd.ExecuteScalar();

                return (Guid)wfid;
            }

            
        }

        internal static string GetNextBookmark(string workflowTypeName, string userName)
        {
            // load from the database based on keys
            using (SqlConnection cnn = new SqlConnection(
                ConfigurationManager.ConnectionStrings[Constants.CONN_PERSISTENCE].ConnectionString))
            {
                SqlCommand cmd = new SqlCommand(
                   String.Format(@"SELECT i.BlockingBookmarks FROM [System.Activities.DurableInstancing].[InstancePromotedProperties] p
                    INNER JOIN [System.Activities.DurableInstancing].[InstancesTable] i
                    on p.InstanceID = i.Id          
                    where p.PromotionName='{0}' and p.Value1 = '{1}' and p.Value2 = '{2}'",
                Constants.PROMOTION_NAME, workflowTypeName, userName),
                    cnn);
                cnn.Open();
                object bookmarks = cmd.ExecuteScalar();
                if (bookmarks != null)
                {
                    var rawBookmark = (string)bookmarks;
                    return rawBookmark.Substring(1, rawBookmark.IndexOf(':') - 1);
                }
                else
                    return null;

                //return (string)bookmarks;
            }


        }
    }
}